# -*- coding: utf-8 -*-

# 获取引擎服务端API的模块
import mod.server.extraServerApi as serverApi
# 获取引擎服务端System的基类，System都要继承于ServerSystem来调用相关函数
ServerSystem = serverApi.GetServerSystemCls()

# 在modMain中注册的Server System类
class TutorialServerSystem(ServerSystem):

    # ServerSystem的初始化函数
    def __init__(self, namespace, systemName):
        # 首先调用父类的初始化函数
        super(TutorialServerSystem, self).__init__(namespace, systemName)
        print "===== TutorialServerSystem init ====="

        # 初始时调用监听函数监听事件
        self.ListenEvent()
        self.ListId=[]
        self.count=-4

    # 监听函数，用于定义和监听函数。函数名称除了强调的其他都是自取的，这个函数也是。
    def ListenEvent(self):
        # 在自定义的ServerSystem中监听引擎的事件ServerChatEvent，回调函数为OnServerChat
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "ServerChatEvent", self, self.OnServerChat)
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), 'ChunkGeneratedServerEvent', self,
                            self.ChunkGeneratedServerEvent)
        self.ListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(),
                            'PlayerInteractServerEvent', self,
                            self.PlayerInteractServerEvent)


        self.DefineEvent("CreateTextEvent")


    # 反监听函数，用于反监听事件，在代码中有创建注册就对应了销毁反注册是一个好的编程习惯，不要依赖引擎来做这些事。
    def UnListenEvent(self):
        self.UnListenForEvent(serverApi.GetEngineNamespace(), serverApi.GetEngineSystemName(), "ServerChatEvent", self, self.OnServerChat)


    def ChunkGeneratedServerEvent(self,args):
        playerIdList=serverApi.GetPlayerList()
        itemDict = {
            'itemName': 'minecraft:wool',
            'count': 64
        }
        comp = serverApi.GetEngineCompFactory().CreateItem(playerIdList[0])
        comp.SpawnItemToPlayerInv(itemDict, playerIdList[0], 1)





    # 监听ServerChatEvent的回调函数
    def OnServerChat(self, args):
        print "==== OnServerChat ==== ", args
        # 生成掉落物品
        # 当我们输入的信息等于右边这个值时，创建相应的物品
        # 创建Component，用来完成特定的功能，这里是为了创建Item物品
        playerId = args["playerId"]
        comp = serverApi.CreateComponent(playerId, "Minecraft", "item")
        comp2 = serverApi.GetEngineCompFactory().CreatePos(playerId)
        comp3 = serverApi.GetEngineCompFactory().CreateDimension(playerId)
        dimensionId=comp3.GetEntityDimensionId()
        playerPos = comp2.GetPos()
        if args["message"] == "钻石剑":
            # 调用SpawnItemToPlayerInv接口生成物品到玩家背包，参数参考《MODSDK文档》
            comp.SpawnItemToPlayerInv({"itemName":"minecraft:diamond_sword", "count":1, 'auxValue': 0}, playerId)
        elif args["message"] == "传送":
            for i in range(1, 5):
                entityId = self.CreateEngineEntityByTypeStr('minecraft:villager_v2',
                                                            ((playerPos[0] + (i*2), playerPos[1] - 1, playerPos[2])),
                                                            (0, 0), dimensionId, True)
                self.ListId.append(entityId)
                print ("CreateEngineEntityByTypeStr" + "----------" + str(entityId) + "------------")
            self.count += 4
            def f():
                textClient = self.CreateEventData()
                textClient["ListId"] = self.ListId
                textClient["count"]=self.count
                self.BroadcastToAllClient("CreateTextEvent", textClient)

            comp4 = serverApi.GetEngineCompFactory().CreateGame(serverApi.GetLevelId())
            print (comp4.AddTimer(1, f))




    def PlayerInteractServerEvent(self, args):
        print "------------PlayerInteractServerEvent-------------"
        playerId = args["playerId"]
        entityID = args["victimId"]
        itemDict = args["itemDict"]
        comp = serverApi.GetEngineCompFactory().CreateDimension(playerId)
        comp2 = serverApi.GetEngineCompFactory().CreateItem(playerId)
        carriedData = comp2.GetPlayerItem(2, 0)
        if itemDict["itemName"]==carriedData["itemName"]:
            if entityID == self.ListId[self.count + 0]:
                # 传送下界
                comp.ChangePlayerDimension(15, (30, 90, -30))
            elif entityID == self.ListId[self.count + 1]:
                # 传送末地
                comp.ChangePlayerDimension(16, (0, 72, 0))
            elif entityID == self.ListId[self.count + 2]:
                # 传送超平坦
                comp.ChangePlayerDimension(17, (0, 6, 0))
            elif entityID == self.ListId[self.count + 3]:
                # 传送超平坦下界
                comp.ChangePlayerDimension(18, (0, 6, 0))




    # 函数名为Destroy才会被调用，在这个System被引擎回收的时候会调这个函数来销毁一些内容
    def Destroy(self):
        print "===== TutorialServerSystem Destroy ====="
        # 调用上面的反监听函数来销毁
        self.UnListenEvent()
